import java.math.BigInteger;

class Factor{
	/**
	 * A binary-search-based square-root finder
	 */
	public static BigInteger sqrt(BigInteger n){
		BigInteger lo = BigInteger.ONE;
		BigInteger hi = n;
		BigInteger m = null;
		while (hi.subtract(lo).compareTo(BigInteger.ONE) > 0)
		{
			m = lo.add(hi).shiftRight(1);
			if (m.multiply(m).compareTo(n) > 0) 
				hi = m; 
			else 
				lo = m;

		}
		return lo;
	}

	public static boolean isSquare(BigInteger n){
		BigInteger s = sqrt(n);
		return s.multiply(s).equals(n);
	}

	/**
	 * Accepts a large positive integer as a parameter and attempts to 
	 * factor it using a certain algorithm
	 */
	public static void main(String[] args){
		BigInteger n = null;
		try{
			n = new BigInteger(args[0]);
		}catch (Exception e){
			System.out.println("Usage: java Factor n");
			return;
		}

		System.out.println("N = " + n);
		BigInteger x = sqrt(n);
		BigInteger y = null;
		System.out.println("Sqrt(N) = " + x);
		
		if (isSquare(n))
		{
			System.out.println("N is a square");
		}

		while(true){
			System.out.print("Testing x = " + x);
			y = x.multiply(x).subtract(n);
			if (isSquare(y)){
				System.out.println(" -> Success");
				System.out.println("The factors are " + x.add(y) + " and " + x.subtract(y));
				return;
			}
			else {
				System.out.println(" -> Failure");
			}
			x = x.add(BigInteger.ONE);
		}
	}
}
