import java.math.BigInteger;

class Factor2{
	/**
	 * A binary-search-based square-root finder
	 */
	public static BigInteger sqrt(BigInteger n){
		BigInteger lo = BigInteger.ONE;
		BigInteger hi = n;
		BigInteger m = null;
		while (hi.subtract(lo).compareTo(BigInteger.ONE) > 0)
		{
			m = lo.add(hi).shiftRight(1);
			if (m.multiply(m).compareTo(n) > 0) 
				hi = m; 
			else 
				lo = m;

		}
		return lo;
	}

	public static boolean isSquare(BigInteger n){
		BigInteger s = sqrt(n);
		return s.multiply(s).equals(n);
	}

	/**
	 * A linear-search-based log finder
	 */
	public static int log(BigInteger n, BigInteger base){
		int c = 1;
		while (base.pow(c).compareTo(n) <= 0) c++;
		return --c;
	}

	/**
	 * Accepts a large positive integer as a parameter and attempts to 
	 * factor it using a certain algorithm
	 */
	public static void main(String[] args){
		BigInteger n = null;
		try{
			n = new BigInteger(args[0]);
		}catch (Exception e){
			System.out.println("Usage: java Factor2 n");
			return;
		}

		BigInteger[] primes = {new BigInteger("2"), new BigInteger("3"), new BigInteger("5"),
			                   new BigInteger("7")};
		int[] upperBounds = new int[primes.length];
		for (int i = 0; i < upperBounds.length; i++){
			upperBounds[i] = log(n, primes[i]);
		}

		BigInteger v = null;
		for (int a0 = 0; a0 < upperBounds[0]; a0++){
			for (int a1 = 0; a1 < upperBounds[1]; a1++){
				for (int a2 = 0; a2 < upperBounds[2]; a2++){
					System.out.println("Testing combination " + String.valueOf(a0) + " " + String.valueOf(a1) + " " + String.valueOf(a2));
					v = primes[0].pow(a0);
					v = v.multiply(primes[1].pow(a1));
					v = v.multiply(primes[2].pow(a2));
					System.out.println("Testing number " + v);
					BigInteger b = n.subtract(v).add(BigInteger.ONE).negate();
					BigInteger c = n;
					System.out.println("Solving square equation with a = 1, b = " + b + ", c = " + c);
					BigInteger d = b.multiply(b).subtract(c.shiftLeft(2));
					BigInteger sqrtD = sqrt(d);
					if (sqrtD.multiply(sqrtD).equals(d)){
						BigInteger x1 = sqrtD.subtract(b);
						BigInteger x2 = sqrtD.add(b).negate();
						if (!x1.testBit(0) && !x2.testBit(0)){
							System.out.println("Found solution:");
							System.out.println("p = " + x1.shiftRight(1).abs());
							System.out.println("q = " + x2.shiftRight(1).abs());
							return;
						}
						
					} else System.out.println("Determinant not square. Failed");
				}
			}
		}
		System.out.println("N = " + n);
		System.out.println("Assuming all prime divisors of phi(N) are <= " + primes[primes.length - 1]);
		

		BigInteger x = sqrt(n);
		BigInteger y = null;
		System.out.println("Sqrt(N) = " + x);
		
		if (isSquare(n))
		{
			System.out.println("N is a square");
		}

		while(true){
			System.out.print("Testing x = " + x);
			y = x.multiply(x).subtract(n);
			if (isSquare(y)){
				System.out.println(" -> Success");
				System.out.println("The factors are " + x.add(y) + " and " + x.subtract(y));
				return;
			}
			else {
				System.out.println(" -> Failure");
			}
			x = x.add(BigInteger.ONE);
		}
	}
}
