import java.math.BigInteger;
import java.util.Date;

class Carmichael{

	public static boolean testCarmichael(BigInteger p){
		// Tests that p is a Carmichael number
		if (testPrimeUsual(p)) return false;
		if (testPrimeFermatAll(p)) return true;
		return false;
	}

	public static boolean testPrimeFermat(BigInteger p, BigInteger a){
		// The test succeeds if a^p = a (mod p)
		return a.modPow(p, p).equals(a);
	}

	public static boolean testPrimeFermatAll(BigInteger p){
		// Runs testPrime for all a < p
		BigInteger a = new BigInteger("2");
		while (!a.equals(p) && testPrimeFermat(p, a))
			a = a.add(BigInteger.ONE);
		return (a.equals(p));
	}

	public static boolean testPrimeUsual(BigInteger p){
		// Tries to divide p with all the numbers up to sqrt(p)
		// assumes p > 2
		BigInteger two = BigInteger.ONE.add(BigInteger.ONE);
		if (p.mod(two).equals(BigInteger.ZERO)) return false;
		BigInteger divisor = two.add(BigInteger.ONE);

		while(divisor.pow(2).compareTo(p) < 0){
			if (p.mod(divisor).equals(BigInteger.ZERO)) return false;
			divisor = divisor.add(two);
		}
		return true;
	}

	public static void main(String[] args){
		BigInteger curr = new BigInteger(args[0]);
		System.out.println("Started on " + (new Date()));
		System.out.println("Start number: " + curr);
		while(true){
			if (testCarmichael(curr))
				System.out.println(curr + "");
			curr = curr.add(BigInteger.ONE);
		}
	}
}
