/* Turn timer on to behold how fast we do */
default(timer, 1);

/* A particular discrete logarithm using pollard rho method */
n=4554650419127;
phiN=eulerphi(n);
g=Mod(2433759521297, n);
x=Mod(866756506244, n);

/* We need to find log_g(x) mod n */

/* Takes a vector [k,l,a] where a=x^k * g^l */
f(v) = if(lift(v[3]) <= ceil(n / 3),\
                   [v[1]+1, v[2], v[3]*x], \
       if(lift(v[3]) < ceil(2*lift(n)/3), \
                   [2*v[1], 2*v[2], v[3]*v[3]], \
         [v[1], (v[2]+1)%(n-1), (v[3]*g)%n]));
ff(v)= f(f(v));


/* The algorithm (Pollard's Rho) */
 {
   ai = [Mod(0, phiN), Mod(0, phiN), Mod(1, n)];
   aii= ai;
   ai = f(ai);
   aii= ff(aii);
   while(ai[3] != aii[3],
      ai = f(ai);
      aii = ff(aii);
   );
   xdeg = lift(aii[1] - ai[1]);
   gdeg = lift(ai[2] - aii[2]);

   /* Here we know that x^xdeg = g^gdeg
      Therefore xdeg*log(x) = gdeg    (mod phi n)
      The problem is we can't take inverse of xdeg here.
      In order to be able to do it we first divide
      xdeg and gdeg by their gcd hoping that it
      will help. [in this case it will. I didn't find any
      clues of how to deal with it in general case] */

   d = gcd(xdeg, gdeg);
   xdeg /= d;
   gdeg /= d;

   /* Return back to modulo arithmetics */
   xdeg = Mod(xdeg, phiN);
   gdeg = Mod(gdeg, phiN);
   logx = lift(gdeg/xdeg);
   logx
 }

test = (g^logx == x);