--- Sissejuhatus kryptoloogiasse 6. ylesanne ---
--- Konstantin Tretjakov (kt@ut.ee)          ---

import Numtheory

--- Part 1 ---
n_1 = 4554650419127
g_1 = 2433759521297
x_1 = 866756506244
-- Theoretically should work, but in practice fails [when using hugs] I don't know why.
y_1 = dlog_rho n_1 g_1 x_1


--- Part 2 ---
n_2 = 7564571872200000001
g_2 = 225454359215363
x_2 = 6269276701300145776

y_2 = dlog_SPH n_2 g_2 x_2

test_2 = if modPow n_2 g_2 y_2 == x_2 then "Success" else "Failure"

test = test_2


--- Discrete Logarithm finding Algorithms ----
--- y = log_g(x) mod n

--- Brute force
dlog_bf n g x = searchDL 1
    where searchDL y
            | modPow n g y == x  = y
            | otherwise          = searchDL (y+1)

--- Silver-Pohlig-Hellman algorithm for finding
--- Works for prime n with low divisors of phi n
dlog_SPH n g x =
    let
        -- We need to find y such that g^y == x mod n
        -- For that we find y mod p^k for each factor p^k of (phi n)

        phi_n = n - 1   -- Euler phi for prime n

        find_y_mod_p_k p k =
            let
               -- Represent y as x_0 + x_1 * p + x_2 * p^2 + ...
               -- and find it digit by digit
               digits = (iterate find_digit []) !! k
               find_digit known_digits =
                   let
                     -- See lecture notes for explanations
                     a i = modPow n g (i * phi_n `div` p)
                     x' = x * (modPow n g (-(polynomial known_digits p))) `mod` n
                     p_d = power p (length known_digits + 1)
                     x_to_find = modPow n x' (phi_n `div` p_d)
                     found_digit = head [ i | i <- [0..p-1], a i == x_to_find ]
                   in known_digits ++ [found_digit]

               -- We found all digits [x_0, x_1, ..]
               -- return the value of the polynomial x_0 + x_1*p + ...
            in polynomial digits p

        -- After we have found y mod p^k for every factor p^k of (phi n)
        -- we use the chinese remainder theorem to find y mod (phi n)
    in
        china [ (find_y_mod_p_k p k, power p k) | (p, k) <- factor (phi_n) ]


--- Pollard-Rho discrete logarithm algorithm
--- Won't work in most cases so stays here just for the history
dlog_rho n g x =
    let
        phi_n = n - 1           -- phi n for prime n

    -- Define a sequence a_i
    -- such that a_0 = 1 and a_{i+1} = f(a_i) where
    -- f(a) = x*a , a*a or g*a with some probability.
    -- Therefore we have a_i = x^k * g^l

        f (k, l, a)
           | a < n `div` 3   = (k+1 `mod` phi_n, l   `mod` phi_n, x * a `mod` n)
           | a < 2*n `div` 3 = (2*k `mod` phi_n, 2*l `mod` phi_n, a * a `mod` n)
           | otherwise       = (k   `mod` phi_n, l+1 `mod` phi_n, g * a `mod` n)

    -- we find a cycle in this sequence therefore obtaining
    --  x^k_1 * g^l_1 = x^k_2 * g^l_2     for some k_1, l_1, k_2, l_2
    -- we then find x (if k_1 <> k_2 which is not very probable)
        rho (a@(k_i, l_i, a_i), b@(k_2i, l_2i, a_2i))
           | a_i /= a_2i   = rho ((f a), (f (f b)))
           | otherwise     = (l * modInverse phi_n r) `mod` phi_n
                             where r' = (k_i - k_2i) `mod` phi_n
                                   l' = (l_2i - l_i) `mod` phi_n
                                   r = r' `div` d
                                   l = l' `div` d
                                   d = gcd r' l'
                                      -- if r = 0 the algorithm failed
    in rho ((f (0, 0, 1)), (f (f (0, 0, 1))))
